
#ifndef H_COMMON
#define H_COMMON

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/socket.h>

// Debugging stuff

void ts_debug_i(int level, char *file, int line, char *fmt, ...);

#define ts_debug(level, fmt, args...) ts_debug_i(level, __FILE__, __LINE__, fmt, ##args)
#define ts_error(fmt, args...) ts_debug(-1, fmt, ##args)
#define ts_debug_1(fmt, args...) ts_debug(1, fmt, ##args)
#define ts_debug_2(fmt, args...) ts_debug(2, fmt, ##args)
#define ts_debug_3(fmt, args...) ts_debug(3, fmt, ##args)
#define ts_debug_4(fmt, args...) ts_debug(4, fmt, ##args)
#define ts_fatal(fmt, args...) (ts_error(fmt, ##args), exit(-1))

// SOL_TESLA is the pseudo-level applications can use to talk to TESLA
// via setsockopt

#define SOL_TESLA 817
#define SO_TESLA_HELLO 1

#define SOL_TESLA_IOCTL 818

typedef struct {
    const char *target;
    int optname;
    const void *optval;
    socklen_t optlen;
    void *retval;
    socklen_t *retlen;
} ts_ioctl_s;

// A ts_ioctl is just a setsockopt in disguise.  This avoids having to
// override ioctl (varargs are tricky) although we may change this in
// the future.

#define ts_ioctl(fd, target, optname, optval, optlen, retval, retlen) ({ \
    ts_ioctl_s io = { target, optname, optval, optlen, retval, retlen }; \
    setsockopt(fd, SOL_TESLA_IOCTL, 0, &io, sizeof io); \
})

#ifdef __cplusplus
};
#endif

#endif
