
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "tesla.h"

const char *msg_types[] = {
    "HELLO",
    "SOCKET",
    "CONNECT",
    "FH_FOLLOWS",
    "ACK",
    "NAK",
    "BIND",
    "LISTEN",
    "ACCEPT",
    "GETSOCKOPT",
    "SETSOCKOPT",
    "GETPEERNAME",
    "GETSOCKNAME"
};

/* Debugging support. */

static int debug_level = 0;
void ts_debug_i(int level, char *file, int line, char *fmt, ...)
{
    static int run_yet = 0;
    static FILE* debug_file = 0;
    static int disable = 0;

    if (!run_yet)
    {
	char *value = getenv("TS_DEBUG");

	debug_level = value ? atoi(value) : 0;
	if (debug_level < 0)
	    debug_level = 0;

	value = getenv("TS_DEBUG_FILE");
	if (value)
	    debug_file = fopen(value, "a");

	run_yet = 1;

	if (debug_file)
	{
	    fprintf(debug_file, "\n**********\n\n");
	    ts_debug_2("Writing debug output to \"%s\"", value);
	}
	else
	{
	    debug_file = stderr;
	    if (value)
		ts_error("Unable to open debug file \"%s\"", value);
	}

	ts_debug_2("Debug level set to %d", debug_level);
    }

    if (debug_level >= level)
    {
	va_list ap;
	va_start(ap, fmt);

	if (!disable) {
	    ++disable;
	    fprintf(debug_file, "[%-18s:%3d] ", file, line);
	    vfprintf(debug_file, fmt, ap);
	    fprintf(debug_file, "\n");
	    fflush(debug_file);
	    --disable;
	}
    }
}
