#!/usr/bin/perl

use IO::Socket::INET;
use strict;

my $chunksize = shift @ARGV || 65536;
print "Chunk size is $chunksize\n";

my $oneway = 1;

my $cnt = 0;
sub next_char {
    return chr(int(ord("A") + $cnt++%26));
}

sub do_read {
    my ($sock, $which) = @_;

    my ($bytes, $len, $got);

    # Read length
    $got = $sock->read($bytes, 8);
    die "$which: Didn't get whole length" unless ($got == 8);
    if ($bytes =~ /\D/) {
	die "$which: Not synchronized - wanted length";
    }

    $len = $bytes;
    $len =~ s/^0+//;
    my $foo = int (rand() * $chunksize);
    die "$which: Length is not what I expected" unless ($foo == $len);
    my $got = $sock->read($bytes, $len);
    print "$which: got $got bytes as data\n";
    die "$which: Didn't get whole packet" unless ($got == $len);

    my $ch = "";
    $ch .= next_char() foreach (1..$len);

    die "$which: Read wrong character" unless ($ch eq $bytes);
}

sub do_write {
    my ($sock, $which) = @_;

    # Write length
    my $len = int(rand() * $chunksize);

    my $ch = "";
    $ch .= next_char() foreach (1..$len);

    print "$which: writing $len bytes\n";
    $sock->printf("%08d%s",$len,$ch);
}

sub go {
    my ($sock, $port, $server) = @_;

    srand(81779);

    $sock->autoflush(1);

    my $skipfirst = $server;
    my $which = $server ? "server" : "client";

    $SIG{PIPE} = sub { print "$which: got SIGPIPE\n"; exit(1); };

    if ($oneway) {

	while (1) {
	    if ($server) {
		do_write($sock, $which);
	    } else {
		do_read($sock, $which);
	    }
	}

    } else {

	while (1) {
	    if ($skipfirst) {
		$skipfirst = 0;
	    } else {
		do_read($sock, $which);
	    }
	    do_write($sock, $which);
	}

    }
}

if (@ARGV == 1) {
    # Run client
    my $port = shift @ARGV;
    print "Client port is $port\n";

    my $sock = new IO::Socket::INET(Proto => "tcp",
				    PeerAddr => "localhost",
				    PeerPort => $port) || die "Client is unable to connect: $!";

    go($sock, $port, 0);
    exit;
}

my $port = 8000;
my $sock = undef;

while (!defined($sock)) {
    ++$port;
    print "Trying port $port\n";
    $sock = new IO::Socket::INET(Listen => 5,
				 Proto => "tcp",
				 LocalAddr => "localhost",
				 LocalPort => $port,
				 ReuseAddr => 1);
}

if (!fork()) {
    $sock->close;
    exec "perl $0 $chunksize $port" || die "Unable to re-exec $0";
}

print "Server port is $port\n";
my $cli = $sock->accept || die "Unable to accept";
go($cli, $port, 1);
wait;
exit;

